;;**************************************************************************
;;Source file         : Checkkey.sc
;;Submitted by        : Tom Skinner  73727,1655
;;Version             : 2.0
;;Date                : 1/1/93
;;Description         : Checks whether a record is unique and returns
;;                      true or false.  This procedure will not work
;;                      on the master record of a table with embedded
;;                      records.  I use it primarily for editing lookup
;;                      tables.  For a procedure that deals with all
;;                      possible circumstances see OkKey4.sc by Zenreich
;;                      and Kocis.
;;**************************************************************************
;;Checking for duplicate records in Paradox is more difficult than it should
;;be.  By now Borland should have provided a function that returns TRUE or
;;FALSE depending on whether the key is unique.  Until that day you can use
;;pCheckKey() as long as you are dealing with a single record in a single
;;table.  All of my lookup tables qualify.  In most of my applications
;;there are a couple of tables that need extensive error checking and I
;;write custom procedures to deal with these tables.  Then there are a
;;dozen or so lookup tables that have State Abbreviations, Product Codes,
;;Warehouse Codes, etc.  I use pCheckKey() for these tables.

;;The procedure is very simple.  I copy the record to an array, delete the
;;record, copy the record from the array and try to lock it.  If successful
;;I return TRUE otherwise FALSE.  If TRUE is returned the record may have
;;flown away so I LOCATE it and then RETURN.  If False is returned then I'm
;;still on the record so I just LOOP.

;;There are two other procedures used in my code pWorking() and pError_Msg().
;;pWorking() just puts a working message at the top of the screen.  It takes
;;one argument, the text that appears below the word WORKING.

;;pError_Msg() does the same thing but the colors are white on red so that
;;the user is alerted.  It also beeps.

;;My naming convention is very simple.
;;p = Procedure
;;a = Array
;;v = Variable
;;The letter always preceeds the name.

;;The example is for a lookup table with two fields.
;;"Code"                      :  "A5*"
;;"Description"               :  "A20"

;;Since this is a sample of a pCheckKey() I did not include code for
;;canceling the record, field view or help.

;;These procedures work with Paradox for DOS.  If using version 4 then they
;;may need modification unless you are in COMPATIBLE MODE.  I always use
;;COMPATIBLE MODE.  I don't like STANDARD MODE.  When I want the Windows-like
;;features of STANDARD MODE I use Paradox for Windows.

;;If you use these procedures substitute the your library name for the
;;variable vLibname or assign vLibname a value.
;;**************************************************************************

;;This is a sample of an INSERT record procedure.  Prior to this procedure
;;I have already checked for table locks and inserted a blank record.

WHILE TRUE
    CURSOR NORMAL
    PROMPT FORMAT("W80,AC","INSERT MODE"),
    FORMAT("W80,AC","[Esc]=Cancel          [F2]=Save          [F5]=Edit Field")
    WAIT RECORD
    UNTIL "F1","F2","F5","Esc","Dos","DosBig"
    SWITCH
         CASE RETVAL = "F1"  : ;;Help proc
         CASE RETVAL = "F5"  : ;;Fieldview proc
         CASE RETVAL = "Esc" : ;;Cancel proc
         CASE RETVAL = "F2"  : ;;Save
             pWorking("Checking record")
             IF ISBLANK([Code]) THEN
                 pError_msg("Please assign a unique code for this record")
                 LOOP
             ENDIF
             pCheckKey()
             IF RETVAL THEN
                Moveto [Code]
                Locate aCheckKey["Code"]
                RETURN
             ENDIF
             LOOP
    ENDSWITCH
ENDWHILE
         
;WPBreak==================================================
MESSAGE "Writing procedure pCheckKey"
;=========================================================
PROC pCheckKey()
;=========================================================
; RETURNS True if key is ok, False if key already exists
  CURSOR OFF
  IF NKeyFields(TABLE()) = 0 THEN
    RETURN TRUE
  ENDIF

  CopyToArray aCheckKey
  DEL
  Ins
  CopyFromArray aCheckKey
  LockRecord
  IF RETVAL THEN
    UnLockRecord
    RETURN TRUE
  ELSE
    pError_msg("This is a duplicate")
    RETURN FALSE
  ENDIF
ENDPROC
WRITELIB vLibName  pCheckKey
RELEASE PROCS pCheckKey

;WPBreak=====================================================
MESSAGE "PROC pWorking(vPrompt)"
;============================================================
PROC pWorking(vPrompt)
;============================================================
;============================================================
  CURSOR OFF
  IF MONITOR()="Color" THEN
    STYLE ATTRIBUTE 159
  ELSE
    STYLE REVERSE, BLINK
  ENDIF
  @ 0,0 ?? FORMAT("W80,AC","* * * * *     W O R K I N G     * * * * *")
  IF MONITOR()="Color" THEN
    STYLE ATTRIBUTE 31
  ELSE
    STYLE REVERSE
  ENDIF
  @ 1,0 ?? FORMAT("W80,AC",vPrompt)
  STYLE
ENDPROC

WRITELIB vLibName pWorking
RELEASE PROCS pWorking

;WPBreak========================
MESSAGE "Writing procedure pError_msg(vError_msg)"
;===============================
PROC pError_Msg(vError_msg)
;============================
  IF MONITOR()="Color" THEN
    STYLE ATTRIBUTE 79
  ELSE
    STYLE REVERSE
  ENDIF
  @ 0,0 ?? FORMAT("W80,AC",vError_msg)
  @ 1,0 ?? FORMAT("W80,AC","Press any key")
  STYLE

  WHILE CHARWAITING()
    RETVAL = GETCHAR()
  ENDWHILE
  WHILE NOT CHARWAITING()
    BEEP BEEP
    SLEEP 1000
  ENDWHILE
  RETVAL = GETCHAR()

  @0,0 CLEAR EOL
  @1,0 CLEAR EOL
ENDPROC

WRITELIB vLibName pError_Msg
RELEASE PROCS     pError_Msg
